import {Component, Input} from '@angular/core';
import {
  MatCell,
  MatCellDef,
  MatColumnDef,
  MatHeaderCell,
  MatHeaderRow,
  MatHeaderRowDef,
  MatRow, MatRowDef, MatTable, MatTableDataSource
} from "@angular/material/table";
import {MatFormField, MatLabel} from "@angular/material/form-field";
import {MatIcon} from "@angular/material/icon";
import {MatIconButton} from "@angular/material/button";
import {MatInput} from "@angular/material/input";
import {MatOption} from "@angular/material/autocomplete";
import {MatSelect} from "@angular/material/select";
import {NgForOf, NgIf} from "@angular/common";
import {ReactiveFormsModule} from "@angular/forms";
import {AbstractInteraction} from "../../../project/game-model/interactions/AbstractInteraction";
import {GameModel} from "../../../project/game-model/GameModel";
import {InventoryCondition} from "../../../project/game-model/interactions/condition/InventoryCondition";
import {
  InventoryItemgroupCondition
} from "../../../project/game-model/interactions/condition/InventoryItemgroupCondition";
import {CharacterDependency} from "../../../project/game-model/interactions/CharacterDependency";
import {InventoryItemCondition} from "../../../project/game-model/interactions/condition/InventoryItemCondition";
import {Item} from "../../../project/game-model/inventory/Item";
import {Interaction} from "../../../project/game-model/interactions/Interaction";
import {InventoryAction} from "../../../project/game-model/interactions/actions/InventoryAction";
import {InventoryItemGroupAction} from "../../../project/game-model/interactions/actions/InventoryItemgroupAction";
import {InventoryItemAction} from "../../../project/game-model/interactions/actions/InventoryItemAction";

@Component({
  selector: 'app-item-action-editor',
  templateUrl: './item-action-editor.component.html',
  styleUrl: './item-action-editor.component.scss'
})
export class ItemActionEditorComponent {

  @Input() interaction: Interaction | undefined
  @Input() gameModel: GameModel | undefined
  @Input() group: boolean = false;

  actionDatasource: MatTableDataSource<InventoryAction> = new MatTableDataSource();
  displayedColumns: string[] = ["item", "delta", "edit", "delete"];
  editedAction: InventoryAction | undefined
  addedAction: InventoryAction | undefined

  ngOnInit() {
    this.actionDatasource.data = this.interaction!.inventoryActions;
  }

  addAction() {
    if(this.group) {
      this.addedAction = new InventoryItemGroupAction(CharacterDependency.NONE, 0,  undefined);
    } else {
      this.addedAction = new InventoryItemAction(CharacterDependency.NONE, 0,  undefined);
    }

    this.editedAction = this.addedAction;

    const displayedColumns = this.actionDatasource.data;
    displayedColumns.push(this.addedAction);
    this.actionDatasource.data = displayedColumns;
  }

  editAction(action: InventoryAction) {
    this.editedAction = action;
  }

  finishEditing() {
    if(this.editedAction == undefined) {
      return;
    }

    if(this.editedAction.validate()) {
      if(this.addedAction != undefined) {
        this.interaction!.addAction(this.addedAction);
        this.addedAction = undefined;
      }
      this.editedAction = undefined;
    }
  }

  deleteAction(deletedAction: InventoryAction) {
    this.interaction!.removeAction(deletedAction);
    this.actionDatasource.data = this.actionDatasource.data.filter(action => action !== deletedAction);
  }

  actionAsItemAction(action: InventoryAction) {
    if(action instanceof InventoryItemAction) {
      return action;
    } else {
      return undefined;
    }
  }

  actionAsItemgroupAction(action: InventoryAction) {
    if(action instanceof InventoryItemGroupAction) {
      return action;
    } else {
      return undefined;
    }
  }

  onSelectedItemChange(action: InventoryItemAction, item: Item) {
    (action as InventoryItemAction).item = item;
  }
}
