import {Character} from "../characters/Character";
import {Condition} from "./condition/Condition";
import {Action} from "./actions/Action";
import {AbstractInteraction} from "./AbstractInteraction";
import {ScriptAccountCondition} from "../gamesystems/conditions/ScriptAccountCondition";
import {ScriptAccountAction} from "../gamesystems/actions/ScriptAccountAction";
import {InventoryCondition} from "./condition/InventoryCondition";
import {InventoryItemAction} from "./actions/InventoryItemAction";

export class Interaction extends AbstractInteraction{

  actions: Action[] = []


  constructor(sourceCharacter: Character, targetCharacter: Character | undefined, interactionLabel: string) {
    super(sourceCharacter, targetCharacter, interactionLabel);
  }

  equals(other: AbstractInteraction): boolean {
    if(!(other instanceof Interaction)) {
      return false;
    }

    const equealCharacters = this.sourceCharacter === other.sourceCharacter && this.targetCharacter === other.targetCharacter;
    const equalLabels = this.interactionLabel === other.interactionLabel;
    const equalConditions = this.conditions.every(condition => other.conditions.includes(condition)) && other.conditions.every(condition => this.conditions.includes(condition));

    const equalActions = this.actions.every(action => other.actions.includes(action) && other.actions.every(action => this.actions.includes(action)));

    return equealCharacters && equalLabels && equalConditions && equalActions;
  }

  validate(requieredCharacter: Character): boolean {
    const validCharacters = this.sourceCharacter == requieredCharacter || this.targetCharacter == requieredCharacter;
    const validLabel = this.interactionLabel.length > 0;
    //Todo: Check for contradicting conditions as well as double conditions and actions

    return validCharacters && validLabel;
  }

  get scriptAccountActions() : ScriptAccountAction[]{
    return this.actions.filter(condition => condition instanceof ScriptAccountAction).map(condition => condition as ScriptAccountAction)
  }

  get inventoryActions(): InventoryItemAction[] {
    return this.actions.filter(action => action instanceof InventoryItemAction).map(action => action as InventoryItemAction);
  }

  addAction(action: Action) {
    this.actions.push(action);
  }

  removeAction(deletedAction: Action) {
    this.actions = this.actions.filter(action => action !== deletedAction);
  }
}
